package java.security;

import java.io.ByteArrayInputStream;
import sun.misc.IOUtils;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;
import java.util.Hashtable;
import java.io.ObjectInputStream;
import java.security.cert.CertificateEncodingException;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.util.ArrayList;
import java.security.cert.X509Certificate;
import java.security.cert.Certificate;
import sun.security.util.Debug;
import java.io.Serializable;

public final class UnresolvedPermission extends Permission implements Serializable
{
    private static final long serialVersionUID = -4821973115467008846L;
    private static final Debug debug;
    private String type;
    private String name;
    private String actions;
    private transient Certificate[] certs;
    private static final Class[] PARAMS0;
    private static final Class[] PARAMS1;
    private static final Class[] PARAMS2;
    
    public UnresolvedPermission(String type, String name, String actions, Certificate[] array) {
        super(type);
        if (type == null) {
            throw new NullPointerException("type can't be null");
        }
        this.type = type;
        this.name = name;
        this.actions = actions;
        if (array != null) {
            int i = 0;
            while (i < array.length) {
                if (!(array[i] instanceof X509Certificate)) {
                    this.certs = (Certificate[])array.clone();
                    break;
                }
                ++i;
            }
            if (this.certs == null) {
                int j = 0;
                int n = 0;
                while (j < array.length) {
                    ++n;
                    while (j + 1 < array.length && ((X509Certificate)array[j]).getIssuerDN().equals(((X509Certificate)array[j + 1]).getSubjectDN())) {
                        ++j;
                    }
                    ++j;
                }
                if (n == array.length) {
                    this.certs = (Certificate[])array.clone();
                }
                if (this.certs == null) {
                    ArrayList<Object> list = new ArrayList();
                    for (int k = 0; k < array.length; ++k) {
                        list.add(array[k]);
                        while (k + 1 < array.length && ((X509Certificate)array[k]).getIssuerDN().equals(((X509Certificate)array[k + 1]).getSubjectDN())) {
                            ++k;
                        }
                    }
                    list.toArray(this.certs = new Certificate[list.size()]);
                }
            }
        }
        return;
    }
    
    Permission resolve(Permission permission, Certificate[] array) {
        if (this.certs != null) {
            if (array == null) {
                return null;
            }
            int i = 0;
            while (i < this.certs.length) {
                boolean n = false;
                int j = 0;
                while (j < array.length) {
                    if (this.certs[i].equals(array[j])) {
                        n = true;
                        break;
                    }
                    ++j;
                }
                if (n == 0) {
                    return null;
                }
                ++i;
            }
        }
        try {
            Class<Object> class1 = permission.getClass();
            if (this.name == null && this.actions == null) {
                try {
                    return (Permission)class1.getConstructor(UnresolvedPermission.PARAMS0).newInstance(new Object[0]);
                }
                catch (NoSuchMethodException ex) {
                    try {
                        return (Permission)class1.getConstructor(UnresolvedPermission.PARAMS1).newInstance(new Object[] { this.name });
                    }
                    catch (NoSuchMethodException ex2) {
                        return (Permission)class1.getConstructor(UnresolvedPermission.PARAMS2).newInstance(new Object[] { this.name, this.actions });
                    }
                }
            }
            if (this.name != null && this.actions == null) {
                try {
                    return (Permission)class1.getConstructor(UnresolvedPermission.PARAMS1).newInstance(new Object[] { this.name });
                }
                catch (NoSuchMethodException ex3) {
                    return (Permission)class1.getConstructor(UnresolvedPermission.PARAMS2).newInstance(new Object[] { this.name, this.actions });
                }
            }
            else {
                return (Permission)class1.getConstructor(UnresolvedPermission.PARAMS2).newInstance(new Object[] { this.name, this.actions });
            }
        }
        catch (NoSuchMethodException ex4) {
            if (UnresolvedPermission.debug != null) {
                UnresolvedPermission.debug.println("NoSuchMethodException:\n  could not find proper constructor for " + this.type);
                ex4.printStackTrace();
            }
            return null;
        }
        catch (Exception ex5) {
            if (UnresolvedPermission.debug != null) {
                UnresolvedPermission.debug.println("unable to instantiate " + this.name);
                ex5.printStackTrace();
            }
            return null;
        }
    }
    
    @Override
    public boolean implies(Permission permission) {
        return false;
    }
    
    @Override
    public boolean equals(Object o) {
        if (o == this) {
            return true;
        }
        if (!(o instanceof UnresolvedPermission)) {
            return false;
        }
        UnresolvedPermission unresolvedPermission = (UnresolvedPermission)o;
        if (!this.type.equals(unresolvedPermission.type)) {
            return false;
        }
        if (this.name == null) {
            if (unresolvedPermission.name != null) {
                return false;
            }
        }
        else if (!this.name.equals(unresolvedPermission.name)) {
            return false;
        }
        if (this.actions == null) {
            if (unresolvedPermission.actions != null) {
                return false;
            }
        }
        else if (!this.actions.equals(unresolvedPermission.actions)) {
            return false;
        }
        if ((this.certs == null && unresolvedPermission.certs != null) || (this.certs != null && unresolvedPermission.certs == null) || (this.certs != null && unresolvedPermission.certs != null && this.certs.length != unresolvedPermission.certs.length)) {
            return false;
        }
        int n = 0;
        while (this.certs != null && n < this.certs.length) {
            boolean n2 = false;
            int i = 0;
            while (i < unresolvedPermission.certs.length) {
                if (this.certs[n].equals(unresolvedPermission.certs[i])) {
                    n2 = true;
                    break;
                }
                ++i;
            }
            if (n2 == 0) {
                return false;
            }
            ++n;
        }
        int n3 = 0;
        while (unresolvedPermission.certs != null && n3 < unresolvedPermission.certs.length) {
            boolean n4 = false;
            int j = 0;
            while (j < this.certs.length) {
                if (unresolvedPermission.certs[n3].equals(this.certs[j])) {
                    n4 = true;
                    break;
                }
                ++j;
            }
            if (n4 == 0) {
                return false;
            }
            ++n3;
        }
        return true;
    }
    
    @Override
    public int hashCode() {
        int hashCode = this.type.hashCode();
        if (this.name != null) {
            hashCode ^= this.name.hashCode();
        }
        if (this.actions != null) {
            hashCode ^= this.actions.hashCode();
        }
        return hashCode;
    }
    
    @Override
    public String getActions() {
        return "";
    }
    
    public String getUnresolvedType() {
        return this.type;
    }
    
    public String getUnresolvedName() {
        return this.name;
    }
    
    public String getUnresolvedActions() {
        return this.actions;
    }
    
    public Certificate[] getUnresolvedCerts() {
        return (this.certs == null) ? null : ((Certificate[])this.certs.clone());
    }
    
    @Override
    public String toString() {
        return "(unresolved " + this.type + " " + this.name + " " + this.actions + ")";
    }
    
    @Override
    public PermissionCollection newPermissionCollection() {
        return new UnresolvedPermissionCollection();
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        objectOutputStream.defaultWriteObject();
        if (this.certs == null || this.certs.length == 0) {
            objectOutputStream.writeInt(0);
        }
        else {
            objectOutputStream.writeInt(this.certs.length);
            for (int i = 0; i < this.certs.length; ++i) {
                Certificate certificate = this.certs[i];
                try {
                    objectOutputStream.writeUTF(certificate.getType());
                    byte[] encoded = certificate.getEncoded();
                    objectOutputStream.writeInt(encoded.length);
                    objectOutputStream.write(encoded);
                }
                catch (CertificateEncodingException ex) {
                    throw new IOException(ex.getMessage());
                }
            }
        }
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        Hashtable hashtable = null;
        Object o = null;
        objectInputStream.defaultReadObject();
        if (this.type == null) {
            throw new NullPointerException("type can't be null");
        }
        int int1 = objectInputStream.readInt();
        if (int1 > 0) {
            hashtable = new Hashtable(3);
            o = new ArrayList((int1 > 20) ? 20 : int1);
        }
        else if (int1 < 0) {
            throw new IOException("size cannot be negative");
        }
        for (int i = 0; i < int1; ++i) {
            String utf = objectInputStream.readUTF();
            CertificateFactory instance;
            if (hashtable.containsKey(utf)) {
                instance = (CertificateFactory)hashtable.get(utf);
            }
            else {
                try {
                    instance = CertificateFactory.getInstance(utf);
                }
                catch (CertificateException ex) {
                    throw new ClassNotFoundException("Certificate factory for " + utf + " not found");
                }
                hashtable.put(utf, instance);
            }
            ByteArrayInputStream inStream = new ByteArrayInputStream(IOUtils.readExactlyNBytes(objectInputStream, objectInputStream.readInt()));
            try {
                o.add(instance.generateCertificate(inStream));
            }
            catch (CertificateException ex2) {
                throw new IOException(ex2.getMessage());
            }
            inStream.close();
        }
        if (o != null) {
            this.certs = (Certificate[])o.toArray(new Certificate[int1]);
        }
        return;
    }
    
    static {
        debug = Debug.getInstance("policy,access", "UnresolvedPermission");
        PARAMS0 = new Class[0];
        PARAMS1 = new Class[] { String.class };
        PARAMS2 = new Class[] { String.class, String.class };
    }
}
